/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package data;

import java.io.Serializable;
import java.util.List;

/**
 * A representation of a concept term, i.e. a term that is extracted
 * as relevant for a certain concept. Apart from the string representation
 * of its term, a weight value is present, denoting the relevance degree
 * of this term with the given concept.
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 */
public class CTerm implements Comparable<CTerm>,Serializable{
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;
	
	/** The instance. */
	private String instance;
	
	/** The concept. */
	private String concept;
	
	/** The weight. */
	private double weight;
	
	/** The default weight. */
	private double DEFAULT_WEIGHT = 1.0;
	
	/** The rules. */
	private List<CTerm> rules;
	
	/** The positive. */
	private boolean positive=true;
	
	/** The Constant SEPARATOR. */
	private static final String SEPARATOR = "#";
	
	/** The Constant PARSE_FAILURE_MSG. */
	private static final String PARSE_FAILURE_MSG = "Failed to parse serialized form for CTerm";
	
	/** The null instance. */
	private String NULL_INSTANCE = "#NI#";
	
	/**
	 * Instantiates a new concept term.	 	
	 *
	 * @param concepts the concepts
	 * @param weight the weight
	 * @param positive if the concept (preference) is positive
	 */
	public CTerm(List<CTerm> concepts, double weight,boolean positive) {
		super();
		this.weight = weight;
		this.positive = positive;		
		this.rules = concepts;
	}
	
	/**
	 * Instantiates a new concept-term.
	 *
	 * @param concept the concept
	 * @param weight the concept weight
	 * @param positive if the concept (preference) is positive
	 */
	public CTerm(String concept, double weight, boolean positive) {		
		super();
		this.weight = weight;
		this.positive = positive;		
		this.concept = concept;
		this.instance = NULL_INSTANCE;
	}
	
	/**
	 * Instantiates a new concept-term.
	 *
	 * @param instance the instance
	 * @param concept the concept
	 * @param weight the concept weight
	 * @param positive if the concept (preference) is positive
	 */
	public CTerm(String instance, String concept, double weight, boolean positive) {		
		super();
		this.weight = weight;
		this.positive = positive;		
		this.concept = concept;
		this.instance = instance;
	}
	
	/**
	 * Instantiates a new concept-term.
	 *
	 * @param concept the concept
	 */
	
	public CTerm(String concept) {
		super();
		this.concept = concept;
		this.instance = this.NULL_INSTANCE;
		this.weight = this.DEFAULT_WEIGHT;
		this.positive = true;
	}
	
	/**
	 * Instantiates a new concept-term.
	 *
	 * @param instance the instance
	 * @param concept the concept
	 */
	// Gia na gemizei ton constructor CTerm(List<CTerm> concepts, double weight,boolean positive) me concepts-> instance:concept
	public CTerm(String instance, String concept) {
		super();
		this.concept = concept;
		this.instance = instance;
		this.weight = this.DEFAULT_WEIGHT;
		this.positive = true;
	}
	
	/**
	 * Instantiates a new concept-term from a String.
	 *
	 * @param serialized the serialised string
	 * @param print whether to print out or not
	 */
	public CTerm(String serialized, boolean print){
		super();
		String[] parts = serialized.split(SEPARATOR);
		if (parts.length != 2) throw new IllegalArgumentException(PARSE_FAILURE_MSG + " " + serialized);
		this.instance = parts[0].trim();
		this.weight = Double.parseDouble(parts[1]);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return instance+SEPARATOR+weight;
	}
	
	/**
	 * Gets the concept.
	 *
	 * @return the concept
	 */
	public String getConcept(){
		return this.concept;
	}
	
	/**
	 * Gets the concepts.
	 *
	 * @return the concepts
	 */
	public List<CTerm> getConcepts(){
		return this.rules;
	}
	
	/**
	 * Gets the single instance of CTerm.
	 *
	 * @return single instance of CTerm
	 */
	public String getInstance(){
		return this.instance;
	}
	
	/**
	 * Sets the string representation of the term.
	 * 
	 * @param token the new string representation of the term
	 */
	public void setToken(String token) {
		this.instance = token;
	}

	/**
	 * Sets the concept term weight.
	 * 
	 * @param weight the new concept term weight
	 */
	public void setWeight(double weight) {
		this.weight = weight;
	}

	/**
	 * Increases weight by value <code>addedValue</code>.
	 * 
	 * @param addedValue the added value
	 */
	public void increaseWeight(double addedValue) {
		this.weight += addedValue;
	}
	
	/**
	 * Gets the string representation of the term.
	 * 
	 * @return the string representation of the term
	 */
	public String getToken() {
		return instance;
	}

	/**
	 * Gets the concept weight.
	 * 
	 * @return the concept weight
	 */
	public double getWeight() {
		return weight;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(CTerm otherTerm) {
		if (otherTerm==null) return -1;
		if (weight>otherTerm.getWeight()){
			return 1;
		} else if (weight<otherTerm.getWeight()){
			return -1;
		} else {
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof CTerm)) return false;
		CTerm otherTerm = (CTerm)obj;
		if (instance.equals(otherTerm.getToken())){
			return true;
		} else {
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return instance.hashCode();
	}
	
	/**
	 * Gets if the concept-term is positive or negative.
	 *
	 * @return true if positive
	 */
	public boolean getPositive(){
		return this.positive;
	}
	
	/**
	 * Gets the rules.
	 *
	 * @return the rules
	 */
	public List<CTerm> getRules(){		
		return this.rules;
	}

}
