/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package data;

import java.util.Vector;

/**
 * The structure representing an ontology concept. A concept object
 * consists of the following:
 * i) Concept name, i.e. the string corresponding to the name of the
 * concept in the ontology.
 * ii) A vector containing of terms the are associated with the particular concept.
 * iii) A weight value assigned to the concept denoting its importance
 * in a pool of concepts.
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 */
public class Concept {
	
	/** The name. */
	private String name;
	
	/** The instance. */
	private String instance;
	
	/** The terms. */
	private Vector<CTerm> terms;
	
	/** The weight. */
	private double weight;
	
	/** The gain interest. */
	private double gainInterest;
	
	/** The timestamp. */
	private long timestamp;
	
	/** The object id. */
	private String objectId=null;
	
	/**
	 * Instantiates a new ontology concept.
	 * 
	 * @param name the name of the concept
	 * @param terms the terms that are associated with this concept according
	 * to the graph, along with their association degree
	 * @param weight the weight of the concept
	 */
	public Concept(String name, Vector<CTerm> terms, double weight) {
		super();
		this.name = name;
		this.terms = terms;
		this.weight = weight;
	}
	
	/**
	 * Instantiates a new concept.
	 *
	 * @param name the concept name
	 * @param weight the concept weight
	 */
	public Concept(String name, double weight) {
		this.name = name;		
		this.weight = weight;
		
	}
	
	/**
	 * Instantiates a new concept.
	 *
	 * @param name the concept name
	 * @param weight the concept weight
	 * @param gi the GAIN interest over the concept
	 * @param tsTime the timestamp
	 */
	public Concept(String name, double weight,double gi,long tsTime) {
		this.name = name;		
		this.weight = weight;
		this.gainInterest = gi;
		this.timestamp = tsTime;
	}
	
	/**
	 * Instantiates a new concept.
	 *
	 * @param name the concept name
	 * @param weight the concept weight
	 * @param gi the GAIN interest over the concept
	 * @param tsTime the timestamp
	 * @param objectId the object id
	 */
	public Concept(String name, double weight,double gi,long tsTime,String objectId) {
		this.name = name;		
		this.weight = weight;
		this.gainInterest = gi;
		this.timestamp = tsTime;
		this.objectId=objectId;
	}
	
	/**
	 * Instantiates a new concept.
	 *
	 * @param name the concept name
	 * @param instance the instance
	 * @param weight the concept weight
	 */
	public Concept(String name, String instance,double weight) {
		this.name = name;		
		this.instance = instance;
		this.weight = weight;
		
	}
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return name+" --> "+terms+" --> "+weight;
	}
	
	/**
	 * Gets the object id.
	 *
	 * @return the object id
	 */
	public String getObjectId(){
		return this.objectId;
	}
	
	/**
	 * Gets the timestamp.
	 *
	 * @return the timestamp
	 */
	public long getTimestamp(){
		return this.timestamp;
	}
	
	
	/**
	 * Gets the GAIN interest.
	 *
	 * @return the GAIN interest
	 */
	public double getGainInterest(){
		return this.gainInterest;
	}
	
	/**
	 * Gets the single instance of Concept.
	 *
	 * @return single instance of Concept
	 */
	public String getInstance(){
		return this.instance;
	}
	
	/**
	 * Gets the name of the concept.
	 * 
	 * @return the name of the concept
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the name of the concept.
	 * 
	 * @param name the new name of the concept
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the concept terms.
	 * 
	 * @return the concept terms
	 */
	public Vector<CTerm> getTerms() {
		return terms;
	}
	
	/**
	 * Sets the concept terms.
	 * 
	 * @param terms the new concept terms
	 */
	public void setTerms(Vector<CTerm> terms) {
		this.terms = terms;
	}
	
	/**
	 * Gets the weight of the concept.
	 * 
	 * @return the weight of the concept
	 */
	public double getWeight() {
		return weight;
	}
	
	/**
	 * Sets the weight of the concept.
	 * 
	 * @param weight the new weight of the concept
	 */
	public void setWeight(double weight) {
		this.weight = weight;
	}

}