/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

/**
 * The Class GAINFullExport. Data structure that represents  
 * input from the GAIN user tracking 
 * and preference learning web service.
 * 
 * @author Pantelis Ieronimakis
 */
public class GAINFullExport implements Serializable{
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -1853651287100898278L;
	
	/** The session id. */
	private long sessionID;
	
	/** The object id. */
	private String objectID;
	
	/** The subobject id. */
	private String subobjectID;
	
	/** The interest. */
	private double interest;
	
	/** The concepts. */
	private HashMap<String, Double> concepts;
	
	/** The decision. */
	private Integer decision;
	
	/** The user id. */
	private String userId;
	
	/**
	 * Instantiates a new GAIN full export.
	 */
	public GAINFullExport(){
		this.sessionID = 0;
		this.objectID = "";
		this.subobjectID = "";
		this.interest = 0;
		this.concepts = new HashMap<String, Double>();
		this.decision = 0 ;
	}
	
	/**
	 * Instantiates a new GAIN full export structure.
	 *
	 * @param sid the session id
	 * @param oid the object id
	 * @param soid the sub-object id
	 * @param inter the interest id
	 * @param concs the (DBPedia) concepts
	 */
	public GAINFullExport(long sid, String oid, String soid, double inter, HashMap<String, Double> concs){
		this.sessionID = sid;
		this.objectID = oid;
		this.subobjectID = soid;
		this.interest = inter;
		this.concepts = concs;
	}
	
	/**
	 * Instantiates a new GAIN full export  structure.
	 *
	 * @param sid the session id
	 * @param oid the object id
	 * @param soid the sub-object id
	 * @param inter the interest id
	 * @param concs the (DBPedia) concepts
	 * @param decision the decision
	 */
	public GAINFullExport(long sid, String oid, String soid, double inter, HashMap<String, Double> concs,Integer decision){
		this.sessionID = sid;
		this.objectID = oid;
		this.subobjectID = soid;
		this.interest = inter;
		this.concepts = concs;
	}
	
	/**
	 * Gets the session id.
	 *
	 * @return the session id
	 */
	public long getSessionID(){
		return this.sessionID;
	}
	
	/**
	 * Gets the decision.
	 *
	 * @return the decision
	 */
	public Integer getDecision(){
		return this.decision;
	}
	
	/**
	 * Sets the decision.
	 */
	public void setDecision(){
        this.decision=1;
	}
	
	/**
	 * Sets the session id.
	 *
	 * @param sid the new session id
	 */
	public void setSessionID(long sid){
		this.sessionID = sid;
	}
	
	/**
	 * Gets the object id.
	 *
	 * @return the object id
	 */
	public String getObjectID(){
		return this.objectID;
	}
	
	/**
	 * Sets the object id.
	 *
	 * @param oid the new object id
	 */
	public void setObjectID(String oid){
		this.objectID = oid;
	}
	
	/**
	 * Gets the subobject id.
	 *
	 * @return the subobject id
	 */
	public String getSubobjectID(){
		return this.subobjectID;
	}
	
	/**
	 * Sets the subobject id.
	 *
	 * @param soid the new subobject id
	 */
	public void setSubobjectID(String soid){
		this.subobjectID = soid;
	}
	
	/**
	 * Gets the interest.
	 *
	 * @return the interest
	 */
	public double getInterest(){
		return this.interest;
	}
	
	/**
	 * Sets the interest.
	 *
	 * @param inter the new interest
	 */
	public void setInterest(double inter){
		this.interest = inter;
	}
	
	/**
	 * Gets the concepts.
	 *
	 * @return the concepts
	 */
	public HashMap<String, Double> getConcepts(){
		return this.concepts;
	}
	
	/**
	 * Sets the concepts.
	 *
	 * @param concs the concs
	 */
	public void setConcepts(HashMap<String, Double> concs){
		this.concepts.clear();
		this.concepts.putAll(concs);
	}
	
	/**
	 * Put concept.
	 *
	 * @param concept the concept
	 * @param degree the degree
	 */
	public void putConcept(String concept, double degree){
		this.concepts.put(concept, degree);
	}
	
	/**
	 * Sets the user id.
	 *
	 * @param uid the new user id
	 */
	public void setUserId(String uid){
		this.userId=uid;
	}
	
	/**
	 * Gets the user id.
	 *
	 * @return the user id
	 */
	public String getUserId(){
		return this.userId;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		String s = this.sessionID + ";" + this.objectID  + ";" + this.subobjectID  + ";" + this.interest  + ";";
		for(Iterator<String> iter = this.concepts.keySet().iterator(); iter.hasNext();){
			String c = iter.next();
			double d = this.concepts.get(c);
			s += c + ":" + d + ";";
		}
		
//		return s.substring(0, s.lastIndexOf(";") - 1);
		return s.substring(0, s.lastIndexOf(";"));
	}
	
//	@Override
//	public boolean equals(Object obj) {
//		if (!(obj instanceof GAINFullExport)) return false;
//		GAINFullExport otherEntry = (GAINFullExport)obj;
//		if (lemma.equals(otherLemma.getLemma()) && type.equals(otherLemma.getType())) 
//			return true;
//		return false;
//	}
	
}
