/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */

package data;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Locale;

/**
 * Represents a composite concept of preference for a particular user. It consists
 * of a set of String objects pointing to ontology concepts and a weight
 * indicating the importance of the concept to the user. Typically a concept is
 * consisted of the following:<br>
 * 1.The ontology concepts included in the composite profile concept. The
 * profile concept can either consist of a single ontology concept or a
 * combination of ontology concepts.<br>
 * 2.The weight of the profile concept.<br>
 * 3.If it is positive (user's selection) or negative (user ignored it).<br>
 * 4.The timestamp (the last time that the concept was derived by the users
 * selections).<br>
 * 5.The frequency that the the profile concept has appeared in the profile.<br>
 * 6.The average degree of the concept as it was derived by the classifier.<br>
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 * 
 */
public class ProfileConcept implements Comparable<ProfileConcept> {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The Constant MEAN_LIFETIME. */
	private static final double MEAN_LIFETIME = 7 * 24 * 60 * 60 * 1000.0;

	/** The Constant lamda. */
	private static final double lamda = 0.00000000000009;

	/** The concept. */
	private String concept = "";

	/** The instance. */
	private String instance = "";

	/** The context string. */
	private String contextString = "";

	/** The concepts. */
	private ArrayList<String> concepts = new ArrayList<String>(0);

	/** The context. */
	private ArrayList<String> context = new ArrayList<String>(0);

	/** The aggregated weight. */
	private double aggregatedWeight = -1.0;

	/** The positive. */
	private boolean positive = true;

	/** The last timestamp. */
	private long lastTimestamp = 0;

	/** The freq. */
	private int freq = 0;

	/** The interest. */
	private double interest = 0;

	/** The object id. */
	private String objectId = "";

	/** The Constant SEPARATOR. */
	private static final String SEPARATOR = "\t";

	/** The Constant PARSE_FAILURE_MSG. */
	private static final String PARSE_FAILURE_MSG = "Failed to parse serialized form for ProfileConcept";

	/**
	 * <i>Null constructor.
	 */
	public ProfileConcept() {

	}

	/**
	 * Constructs a profile concept.
	 * 
	 * @param concept
	 *            the concept
	 * @param freq
	 *            the frequency
	 * @param lastTimestamp
	 *            the last timestamp
	 * @param positive
	 *            the boolean stating whether this profile concept is positive
	 *            or negative
	 * @param aggregatedweight
	 *            the aggregatedweight
	 * @param context
	 *            the context
	 * @param gainInterest
	 *            the gain interest
	 */
	public ProfileConcept(String concept, int freq, long lastTimestamp,
			boolean positive, double aggregatedweight,
			ArrayList<String> context, Double gainInterest) {
		this.concept = concept;
		this.freq = freq;
		this.lastTimestamp = lastTimestamp;
		this.positive = positive;
		this.aggregatedWeight = aggregatedweight;
		this.context = context;
		this.interest = gainInterest;
	}

	/**
	 * Instantiates a new profile concept.
	 * 
	 * @param concepts
	 *            the concepts
	 * @param freq
	 *            the freq
	 * @param lastTimestamp
	 *            the last timestamp
	 * @param positive
	 *            the positive
	 * @param avgDegree
	 *            the avg degree
	 */
	public ProfileConcept(ArrayList<String> concepts, int freq,
			long lastTimestamp, boolean positive, double avgDegree) {
		this.concepts.addAll(concepts);
		this.freq = freq;
		this.lastTimestamp = lastTimestamp;
		this.positive = positive;
		this.aggregatedWeight = avgDegree;
		this.context.add("null");
	}

	/**
	 * Instantiates a new profile concept.
	 * 
	 * @param concepts
	 *            the concepts
	 * @param freq
	 *            the freq
	 * @param lastTimestamp
	 *            the last timestamp
	 * @param positive
	 *            the positive
	 * @param aggregatedweight
	 *            the aggregatedweight
	 * @param context
	 *            the context
	 */
	public ProfileConcept(ArrayList<String> concepts, int freq,
			long lastTimestamp, boolean positive, double aggregatedweight,
			ArrayList<String> context) {
		this.concepts = concepts;
		this.freq = freq;
		this.lastTimestamp = lastTimestamp;
		this.positive = positive;
		this.aggregatedWeight = aggregatedweight;
		this.context = context;
	}

	/**
	 * Instantiates a new profile concept.
	 * 
	 * @param concept
	 *            the concept
	 * @param freq
	 *            the freq
	 * @param lastTimestamp
	 *            the last timestamp
	 * @param positive
	 *            the positive
	 * @param aggregatedweight
	 *            the aggregatedweight
	 * @param context
	 *            the context
	 */
	public ProfileConcept(String concept, int freq, long lastTimestamp,
			boolean positive, double aggregatedweight, ArrayList<String> context) {
		this.concept = concept;
		this.freq = freq;
		this.lastTimestamp = lastTimestamp;
		this.positive = positive;
		this.aggregatedWeight = aggregatedweight;
		this.context = context;
	}

	/**
	 * Instantiates a new profile concept.
	 * 
	 * @param concept
	 *            the concept
	 * @param instance
	 *            the instance
	 * @param freq
	 *            the freq
	 * @param lastTimestamp
	 *            the last timestamp
	 * @param positive
	 *            the positive
	 * @param aggregatedweight
	 *            the aggregatedweight
	 * @param context
	 *            the context
	 */
	public ProfileConcept(String concept, String instance, int freq,
			long lastTimestamp, boolean positive, double aggregatedweight,
			ArrayList<String> context) {
		this.concept = concept;
		this.instance = instance;
		this.freq = freq;
		this.lastTimestamp = lastTimestamp;
		this.positive = positive;
		this.aggregatedWeight = aggregatedweight;
		this.context = context;
	}

	/**
	 * Constructs a <code>ProfileConcept</code> object by an existing
	 * <code>ProfileConcept</code>.
	 * 
	 * @param c
	 *            the existing <code>ProfileConcept</code>.
	 */
	public ProfileConcept(ProfileConcept c) {
		this.concepts.addAll(c.getConcepts());
		this.freq = c.getFreq();
		this.lastTimestamp = c.getLastTimestamp();
		this.aggregatedWeight = c.getWeight();

	}

	/**
	 * Constructs a <code>ProfileConcept</code> from a String.
	 * 
	 * @param serialized
	 *            the serialized
	 */
	public ProfileConcept(String serialized) {
		NumberFormat nf = NumberFormat.getNumberInstance(Locale.getDefault());
		String[] parts = serialized.split("\t");
		if (parts.length != 6)
			throw new IllegalArgumentException(PARSE_FAILURE_MSG + " "
					+ serialized);
		this.lastTimestamp = Long.parseLong(parts[0]);
		try {
			this.aggregatedWeight = nf.parse(parts[2]).doubleValue();
		} catch (ParseException e) {
			e.printStackTrace();
		}
		this.freq = Integer.parseInt(parts[3]);
		this.positive = Boolean.parseBoolean(parts[4]);
		this.contextString = String.valueOf(parts[5]);
		ArrayList<String> concepts = new ArrayList<String>();
		concepts.add(parts[1].substring(1, parts[1].length() - 1));
		this.concepts = concepts;
	}

	/**
	 * Gets the concept.
	 * 
	 * @return the concept
	 */
	public String getConcept() {
		return this.concept;
	}

	/**
	 * Sets the concept.
	 * 
	 * @param c
	 *            the new concept
	 */
	public void setConcept(String c) {
		this.concept = c;
	}

	/**
	 * Sets the instance.
	 * 
	 * @param i
	 *            the new instance
	 */
	public void setInstance(String i) {
		this.concept = i;
	}

	/**
	 * Gets the single instance of ProfileConcept.
	 * 
	 * @return single instance of ProfileConcept
	 */
	public String getInstance() {
		return this.instance;
	}

	/**
	 * Gets the gain interest.
	 * 
	 * @return the gain interest
	 */
	public double getGainInterest() {
		return this.interest;
	}

	/**
	 * Returns a String representation of the <code>ProfileConcept</code>.
	 * 
	 * @return the string representation of the <code>ProfileConcept
	 */
	public String toString() {

		return this.getLastTimestamp() + SEPARATOR + concepts.toString()
				+ SEPARATOR + aggregatedWeight + SEPARATOR + freq + SEPARATOR
				+ positive + SEPARATOR + null + "\n";
	}

	/**
	 * Returns a String representation of a <code>ProfileConcept</code> in the
	 * format that the profile concepts are presented in the <i>Profile</i> tab
	 * of the web interface.
	 * 
	 * @return the string representation of the <code>ProfileConcept
	 */
	public String toStringProfileFormat() {
		return aggregatedWeight + SEPARATOR + positive + SEPARATOR + SEPARATOR
				+ concepts.toString();
	}

	/**
	 * Gets the ontology concepts in the <code>ProfileConcept</code>.
	 * 
	 * @return the ontology concepts
	 */
	public ArrayList<String> getConcepts() {
		return concepts;
	}

	/**
	 * Gets the concepts in rule.
	 * 
	 * @param complexConcept
	 *            the complex concept
	 * @return the concepts in rule
	 */
	public ArrayList<String> getConceptsInRule(ProfileConcept complexConcept) {
		String[] concs = complexConcept.getConcept().split("and");
		ArrayList<String> concepts = new ArrayList<String>();
		for (int i = 0; i < concs.length; i++) {
			String concept = concs[i].trim();
			concepts.add(concept);
		}

		return concepts;

	}

	/**
	 * Sets the ontology concepts for the <code>ProfileConcept</code>.
	 * 
	 * @param concepts
	 *            the new ontology concepts
	 */
	public void setConcepts(ArrayList<String> concepts) {
		this.concepts = concepts;
	}

	/**
	 * Adds an ontology concept.
	 * 
	 * @param concept
	 *            the ontology concept
	 */
	public void addConcept(String concept) {
		this.concepts.add(concept);
	}

	/**
	 * Returns the saved weight of the <code>ProfileConcept</code>.
	 * 
	 * @return the saved weight of the <code>ProfileConcept</code>.
	 */
	public double getWeight() {
		return (aggregatedWeight);
	}

	/**
	 * Calculates and returns the current weight of the
	 * <code>ProfileConcept</code>.
	 * 
	 * @return the current weight.
	 */
	/*
	 * public double getCurrentWeight() { return roundDouble(avgDegree * freq *
	 * Math.exp(-((System.currentTimeMillis()-lastTimestamp)/MEAN_LIFETIME))); }
	 */

	/**
	 * Returns the saved weight of the <code>ProfileConcept</code> with the use
	 * of a normaliser that represents the weight in the [0,1] interval.
	 * 
	 * @return the normalised weight
	 */
	public double getNormalisedWeight() {
		return (this.aggregatedWeight / this.freq);
	}

	/**
	 * Calculates and returns the current weight of the
	 * <code>ProfileConcept</code> with the use of a normaliser that represents
	 * the weight in the [0,1] interval.
	 * 
	 * @param transactionsNumber
	 *            the transactions number
	 * @return the current normalised weight.
	 */
	public double getCurrentNormalisedWeight(int transactionsNumber) {
		return this.getNormalisedWeight()// aggr weight/freq
				* ((Math.exp(-((System.currentTimeMillis() - this.lastTimestamp) / MEAN_LIFETIME))) / (double) transactionsNumber);// time
																																	// decay
	}

	/**
	 * Gets the current normalised weight new version.
	 *
	 * @param transactionsNumber the transactions number
	 * @return the current normalised weight new version
	 */
	public double getCurrentNormalisedWeightNewVersion(int transactionsNumber) {

		System.out.println("TWRINO TIMESTAMP : " + (System.currentTimeMillis())
				+ " MEION " + this.lastTimestamp);
		System.out.println(System.currentTimeMillis() - this.lastTimestamp);
		return this.getNormalisedWeight()// aggr weight/freq
				* ((Math.pow(
						2,
						-((System.currentTimeMillis() - this.lastTimestamp) * lamda))));// time
		// decay
	}
	
	/**
	 * Gets the current normalised weight lamda version.
	 *
	 * @param transactionsNumber the transactions number
	 * @param lamda the lamda
	 * @return the current normalised weight lamda version
	 */
	public double getCurrentNormalisedWeightLamdaVersion(int transactionsNumber,double lamda) {

		System.out.println("TWRINO TIMESTAMP : " + (System.currentTimeMillis())
				+ " MEION " + this.lastTimestamp);
		System.out.println(System.currentTimeMillis() - this.lastTimestamp);
		return this.getNormalisedWeight()// aggr weight/freq
				* ((Math.pow(
						2,
						-((System.currentTimeMillis() - this.lastTimestamp) * lamda))));// time
		// decay
	}
	
	/**
	 * Sets the profile concept's weight.
	 * 
	 * @param w
	 *            the new weight
	 */
	public void setWeight(double w) {
		this.aggregatedWeight = w;
	}

	/**
	 * Checks if the profile concept is positive.
	 * 
	 * @return true, if it is positive
	 */
	public boolean isPositive() {
		return positive;
	}

	/**
	 * Sets the boolean that describes whether the profile concept is positive
	 * or not.
	 * 
	 * @param positive
	 *            the new positive
	 */
	public void setPositive(boolean positive) {
		this.positive = positive;
	}

	/**
	 * Gets the last timestamp.
	 * 
	 * @return the last timestamp
	 */
	public long getLastTimestamp() {
		return lastTimestamp;
	}

	/**
	 * Sets the last timestamp.
	 * 
	 * @param lastTimestamp
	 *            the new timestamp
	 */
	public void setLastTimestamp(long lastTimestamp) {
		this.lastTimestamp = lastTimestamp;
	}

	/**
	 * Gets the frequency.
	 * 
	 * @return the frequency
	 */
	public int getFreq() {
		return freq;
	}

	/**
	 * Sets the frequency.
	 * 
	 * @param freq
	 *            the new frequency
	 */
	public void setFreq(int freq) {
		this.freq = freq;
	}

	/**
	 * Gets the average degree.
	 * 
	 * @param obj
	 *            the obj
	 * @return the average degree
	 */
	/*
	 * public double getAvgDegree() { return avgDegree; }
	 */

	/**
	 * Sets the average degree.
	 * 
	 * @param degree
	 *            the new average degree
	 */
	/*
	 * public void setAvgDegree(double degree) { avgDegree = degree; }
	 */

	/**
	 * Compares the weight of two <code>ProfileConcept</code> objects.
	 * 
	 * @param otherConcept
	 *            the other concept to compare
	 * 
	 * @return 1 if the weight of the first <code>ProfileConcept</code> is
	 *         greater than the weight of the latter. Else 0.
	 */
	// public int compareTo(ProfileConcept_Original otherConcept) {
	// double thisWeight = this.getWeight();
	// if (thisWeight > otherConcept.getWeight()) {
	// return 1;
	// } else if (thisWeight < otherConcept.getWeight()) {
	// return -1;
	// } else {
	// return 0;
	// }
	// }

	/**
	 * Compares two <code>ProfileConcept</code> objects.
	 * 
	 * @param obj
	 *            the object to compare
	 * 
	 * @return true, of the object equals the <code>ProfileConcept</code>.
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof ProfileConcept) {
			if (this.isPositive() != ((ProfileConcept) obj).isPositive()) {
				return false;
			}
			ArrayList<String> otherConcepts = ((ProfileConcept) obj)
					.getConcepts();
			Collections.sort(otherConcepts);
			Collections.sort(this.concepts);
			if (otherConcepts.hashCode() == this.concepts.hashCode()) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		Collections.sort(concepts);
		return new Boolean(positive).hashCode() + concepts.hashCode();
	}

	/**
	 * Utility method that reads profile concepts from a file.
	 * 
	 * @param filename
	 *            : the path to the file that contains the concepts.
	 * 
	 * @return the profile concepts that have been retrieved from the file.
	 */
	public static ArrayList<ProfileConcept> readFromFile(String filename) {
		ArrayList<ProfileConcept> profileConcepts = new ArrayList<ProfileConcept>(
				100);
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(filename));
			String line = null;
			while ((line = reader.readLine()) != null) {

				profileConcepts.add(new ProfileConcept(line));
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				reader.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			;
		}
		return profileConcepts;
	}

	/**
	 * Utility method that writes profile concepts to a file.
	 * 
	 * @param concepts
	 *            the concepts to be written to the file.
	 * @param filename
	 *            the path to the file that the concepts will be written.
	 */
	public static void writeToFile(ArrayList<ProfileConcept> concepts,
			String filename) {
		BufferedWriter writer = null;
		// System.out.println("FIlename:"+filename);
		try {
			writer = new BufferedWriter(new FileWriter(filename, false));

			for (int i = 0; i < concepts.size(); i++) {
				writer.write(concepts.get(i).toString());
				writer.newLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				writer.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			;
		}
	}

	/**
	 * Utility method that writes profile concepts to a file, in the format that
	 * the profile concepts are presented in the <i>Profile</i> tab of the web
	 * interface.
	 * 
	 * @param concepts
	 *            : The concepts to be written to the file.
	 * @param filename
	 *            : the path to the file that the concepts will be written.
	 */
	public static void writeToFileProfileFormat(
			ArrayList<ProfileConcept> concepts, String filename) {
		BufferedWriter writer = null;
		// System.out.println("FIlename:"+filename);
		try {
			writer = new BufferedWriter(new FileWriter(filename, false));

			for (int i = 0; i < concepts.size(); i++) {
				writer.write(concepts.get(i).toStringProfileFormat());
				writer.newLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				writer.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			;
		}
	}

	/**
	 * Rounds a double number into <code>decimalPlace</code> precision.
	 * 
	 * @param targetDBL
	 *            the double number to round.
	 * 
	 * @return the rounded up double value.
	 */
	public static double roundDouble(double targetDBL) {
		int decimalPlace = 4;
		BigDecimal bd = new BigDecimal(targetDBL);
		bd = bd.setScale(decimalPlace, BigDecimal.ROUND_UP);

		return (bd.doubleValue());
	}

	/**
	 * Compares the weight of two <code>ProfileConcept</code> objects.
	 * 
	 * @param otherPConcept
	 *            the other concept to compare
	 * 
	 * @return 1 if the last timestamp of the first <code>ProfileConcept</code>
	 *         is greater than the last timestamp of the latter. Else 0.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(ProfileConcept otherPConcept) {
		if (otherPConcept == null)
			return -1;
		if (this.lastTimestamp > otherPConcept.getLastTimestamp()) {
			return 1;
		} else if (this.lastTimestamp < otherPConcept.getLastTimestamp()) {
			return -1;
		} else {
			return 0;
		}
	}

	/**
	 * Sets the weight.
	 * 
	 * @param weight
	 *            the new weight
	 */
	public void setWeight(Double weight) {
		this.aggregatedWeight = weight;
	}
}
