/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package onto;

import java.util.Set;

import store.OntologyIO;
import lifr.logic.firstorder.KnowledgeBase;
import lifr.util.KRSSFuzzyParser;

/**
 * Class used to retrieve the domain ontology in whole, or specific parts of the
 * ontology. In each case, the input files are converted from the KRSS format to
 * logic rules in Protein syntax.
 * 
 * @author Dorothea Tsatsou
 */
public class OntologyRetriever {

	/**
	 * Retrieve the whole ontology (stored locally) and convert it (from the stored
	 * KRSS format) to a knowledge base of logic rules.
	 * 
	 * @return the knowledge base in logic rules
	 */
	public static final KnowledgeBase getOntology() {
		KnowledgeBase kb = new KnowledgeBase();
		String tbox = OntologyIO.loadTBox();
		KRSSFuzzyParser parser = new KRSSFuzzyParser(tbox);
		kb = parser.getKB();
		return kb;
	}

	/**
	 * Retrieve the mappings of the core ontology (stored locally and in an ontological format) 
	 * to other ontologies and convert it (from the stored KRSS format) to a knowledge base of logic rules.
	 *
	 * @return the ontology mappings
	 */
	public static final KnowledgeBase getOntologyMappings() {
		KnowledgeBase kb = new KnowledgeBase();
		String tbox = OntologyIO.loadMappingsTBox();
		KRSSFuzzyParser parser = new KRSSFuzzyParser(tbox);
		kb = parser.getKB();
		return kb;
	}
	
	/**
	 * Retrieve a subset of the mappings of the core ontology (stored locally and in an ontological format) 
	 * to other ontologies and convert it (from the stored KRSS format) to a knowledge base of logic rules.
	 *
	 * @return the ontology mappings
	 */
	public static final KnowledgeBase getOntologyMappingsSubset(Set<String> concepts) {
		KnowledgeBase kb = new KnowledgeBase();
		String tbox = OntologyIO.loadMappingsSubsetTBox(concepts);
		KRSSFuzzyParser parser = new KRSSFuzzyParser(tbox);
		kb = parser.getKB();
		return kb;
	}
}
