/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>. 
 */
package profile;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import org.eclipse.osgi.internal.signedcontent.Base64;

import store.ProfileIO;
import sys.Config;
import data.ProfileConcept;
import lifr.logic.firstorder.KnowledgeBase;

/**
 * Class used to construct a KRSS user profile based on a set of ontology
 * concepts defining the user. These profile concepts have derived through
 * logging and management of the user transactions. Practically, converts a 
 * Plain User Profile to the final (ontological) profile format. 
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 */
public class UserProfileConstructor {

	/** The user id. */
	private String userID = "";
	
	/** The positive concepts. */
	private ArrayList<ProfileConcept> positiveConcepts;
	
	/** The negative concepts. */
	private ArrayList<ProfileConcept> negativeConcepts;
	
	/** The originalkb. */
	@SuppressWarnings("unused")
	private KnowledgeBase originalkb = null;

	/**
	 * Instantiates a new user profile constructor.
	 */
	public UserProfileConstructor() {

	}

	/**
	 * Constructor of the class, used to instantiate a new user profile.
	 * 
	 * @param ID
	 *            the username.
	 * @param pos
	 *            the profile concepts derived from positive transactions
	 * @param neg
	 *            the profile concepts derived from negative transactions
	 */
	public UserProfileConstructor(String ID, ArrayList<ProfileConcept> pos,
			ArrayList<ProfileConcept> neg) {
		userID = ID;
		positiveConcepts = pos;
		negativeConcepts = neg;
	}

	/**
	 * Construction method that creates a new file (or overwrites an existing
	 * one) which will contain the user profile rule in the KRSS format.
	 * 
	 * @param ontokb
	 *            the ontology.
	 */
	public void constructProfile(KnowledgeBase ontokb) {
		originalkb = ontokb;
		ProfileIO.createUserProfileFile(userID, createKRSSProfile());
	}

	/**
	 * Construction method that creates a new file (or overwrites an existing
	 * one) which will contain the user profile rule in the KRSS format.
	 *
	 * @param ontokb the ontology.
	 * @param lamda the lamda timestamp-related constant
	 */
	public void constructProfile(KnowledgeBase ontokb, Double lamda) {
		originalkb = ontokb;
		ProfileIO.createUserProfileFile(userID, createKRSSProfile(lamda));
	}

	/**
	 * Method that produces a <code>String</code> of the user profile KRSS rule.
	 * 
	 * @return the user profile KRSS rule.
	 */
	private final String createKRSSProfile() {
		String userFile = "(IMPLIES ";
		String posConcepts = "";
		String negConcepts = "";
		String instanceConstructor = "";
		String weightConstructor = "";
		String relatedContentConstructor = "";
		String relatedRuleConstructor = "";
		String relatedConstructor = "(RELATED content rule has_interest)";
		String disjointConstructor = "(DISJOINT user" + userID + " user"
				+ userID + "_dis)\n";

		Integer associationRulesNumber = 0;

		Double max = 0.00;

		HashMap<String, String> instancesBooleanMap = new HashMap<>();
		HashMap<String, Double> instancesWeightsMap = new HashMap<>();

		ArrayList<ProfileConcept> simpleConcepts = new ArrayList<ProfileConcept>();
		ArrayList<ProfileConcept> entities = new ArrayList<ProfileConcept>();
		ArrayList<ProfileConcept> associationRules = new ArrayList<ProfileConcept>();

		String accountID = "YOUTUBE-TEST";

		UserProfileConstructor krss = new UserProfileConstructor();
		String sessionsNumber = krss.getSessionsNumber(accountID, userID);

		DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
				Locale.getDefault());
		europSymbols.setDecimalSeparator('.');
		DecimalFormat df = new DecimalFormat("0.0000", europSymbols);
		df.setMaximumFractionDigits(8);

		for (Iterator<ProfileConcept> iter = positiveConcepts.iterator(); iter
				.hasNext();) {
			ProfileConcept concept = iter.next();
			ArrayList<String> conc = concept.getConcepts();
			if (conc.get(0).contains(":")) {
				instancesBooleanMap.put(conc.get(0).split(":")[0], "false");
				max = concept.getWeight();
				if (concept.getWeight() > max)
					instancesWeightsMap.put(conc.get(0).split(":")[0],
							concept.getWeight());
				else
					instancesWeightsMap.put(conc.get(0).split(":")[0], max);
				entities.add(concept);
			} else if (conc.get(0).contains(" AND ")) {
				associationRules.add(concept);
			} else {
				simpleConcepts.add(concept);
			}
		}

		if ((!positiveConcepts.isEmpty()) && (!negativeConcepts.isEmpty()))
			userFile = userFile + "(AND ";

		if (!positiveConcepts.isEmpty()) {

			posConcepts = posConcepts + "(SOME has_interest (OR ";
			for (Iterator<ProfileConcept> iter = positiveConcepts.iterator(); iter
					.hasNext();) {
				ProfileConcept concept = iter.next();
				ArrayList<String> conc = concept.getConcepts();

				associationRulesNumber = 0;

				conc.trimToSize();
				if (conc.size() > 1) {
					posConcepts = posConcepts.concat("(AND ");
					for (Iterator<String> concElem = conc.iterator(); concElem
							.hasNext();) {

						@SuppressWarnings("unused")
						String currConcept = concElem.next();
					}
					posConcepts = posConcepts.concat(") ");
				} else {
					if (conc.get(0).contains(" AND ")) {
						posConcepts = posConcepts.concat(" profsubconcept"
								+ ++associationRulesNumber);
						weightConstructor += "(WEIGHT "
								+ "profsubconcept"
								+ associationRulesNumber
								+ " "
								+ df.format(concept
										.getCurrentNormalisedWeightNewVersion(Integer
												.parseInt(sessionsNumber)))
								+ ")\n";
					} else if (conc.get(0).contains(":")) {
						String instance = conc.get(0).split(":")[0];

						// WEIGHT Constructor
						if (instancesBooleanMap.get(instance).equalsIgnoreCase(
								"false")) {
							weightConstructor += "(WEIGHT "
									+ "inst_"
									+ instance
									+ " "
									+ df.format(concept
											.getCurrentNormalisedWeightNewVersion(Integer
													.parseInt(sessionsNumber)))
									+ ")\n";
							posConcepts = posConcepts.concat(" inst_"
									+ instance);
							instancesBooleanMap.put(instance, "true");

							// INSTANCE Constructor
							instanceConstructor += "(INSTANCE " + instance
									+ " inst_" + instance + " >= 0.1)\n";

							// RELATED Constructor
							relatedContentConstructor += "(RELATED content "
									+ instance + " has_interest)\n";
							relatedRuleConstructor += "(RELATED rule "
									+ instance + " inRule)\n";
						}
					} else {
						posConcepts = posConcepts.concat(" " + conc.get(0));
						weightConstructor += "(WEIGHT "
								+ conc.get(0)
								+ " "
								+ df.format(concept
										.getCurrentNormalisedWeightNewVersion(Integer
												.parseInt(sessionsNumber)))
								+ ")\n";
					}
				}
			}
			posConcepts = posConcepts + ")) ";
			associationRulesNumber = 0;
		}

		if (!negativeConcepts.isEmpty()) {
			negConcepts = negConcepts + "(ALL has_interest (NOT (OR ";
			for (Iterator<ProfileConcept> enc = negativeConcepts.iterator(); enc
					.hasNext();) {
				ProfileConcept concept = enc.next();
				ArrayList<String> conc = concept.getConcepts();
				conc.trimToSize();
				if (conc.size() > 1) {
					negConcepts = negConcepts.concat("(AND ");
					for (Iterator<String> concElem = conc.iterator(); concElem
							.hasNext();) {
						@SuppressWarnings("unused")
						String currConcept = concElem.next();
					}
					negConcepts = negConcepts.concat(") ");
				} else {
					// negConcepts =
					// negConcepts.concat(linkProperty(conc.firstElement()) +
					// " ");
					// negConcepts = negConcepts.concat(" " +
					// conc.firstElement() + " ");
				}
			}
			negConcepts = negConcepts + "))) ";
		}

		userFile = userFile + posConcepts + negConcepts;
		if ((!positiveConcepts.isEmpty()) && (!negativeConcepts.isEmpty()))
			userFile = userFile + ")";
		userFile = userFile + userID + ")\n";

		for (Iterator<ProfileConcept> iter = associationRules.iterator(); iter
				.hasNext();) {
			associationRulesNumber++;
			ProfileConcept concept = iter.next();
			userFile += "(IMPLIES (AND";
			String[] rules = concept
					.getConcepts()
					.toString()
					.substring(1, concept.getConcepts().toString().length() - 1)
					.split("\\s+");
			for (int i = 0; i < rules.length; i++) {
				userFile += " (SOME inRule " + rules[i] + ")";
				i++;
			}
			userFile += ") profsubconcept" + associationRulesNumber + ")\n";
		}

		// IMPLIES CONSTRUCTORS (Entities)
		for (Iterator<ProfileConcept> iter = entities.iterator(); iter
				.hasNext();) {
			ProfileConcept concept = iter.next();
			ArrayList<String> conc = concept.getConcepts();
			instancesBooleanMap.put(conc.get(0).split(":")[0], "false");
			userFile += "(IMPLIES inst_"
					+ concept.getConcepts().toString().split(":")[0].replace(
							"[", "")
					+ " "
					+ concept.getConcepts().toString().split(":")[1].replace(
							"]", "") + ")" + "\n";

		}
		String userKRSSProfile = userFile + instanceConstructor
				+ disjointConstructor + weightConstructor
				+ relatedContentConstructor + relatedRuleConstructor
				+ relatedConstructor;
		System.out.println(userKRSSProfile);
		return userKRSSProfile;
	}

	/**
	 * Method that produces a <code>String</code> of the user profile KRSS rule.
	 *
	 * @param lamda the timestamp-related lamda constant
	 * @return the user profile KRSS rule.
	 */
	private final String createKRSSProfile(Double lamda) {
		String userFile = "(IMPLIES ";
		String posConcepts = "";
		String negConcepts = "";
		String instanceConstructor = "";
		String weightConstructor = "";
		String relatedContentConstructor = "";
		String relatedRuleConstructor = "";
		String relatedConstructor = "(RELATED content rule has_interest)";
		String disjointConstructor = "(DISJOINT user" + " user" + "_dis)\n";

		Integer associationRulesNumber = 0;

		Double max = 0.00;

		HashMap<String, String> instancesBooleanMap = new HashMap<>();
		HashMap<String, Double> instancesWeightsMap = new HashMap<>();

		ArrayList<ProfileConcept> simpleConcepts = new ArrayList<ProfileConcept>();
		ArrayList<ProfileConcept> entities = new ArrayList<ProfileConcept>();
		ArrayList<ProfileConcept> associationRules = new ArrayList<ProfileConcept>();

		String accountID = "YOUTUBE-TEST";

		UserProfileConstructor krss = new UserProfileConstructor();
		String sessionsNumber = krss.getSessionsNumber(accountID, userID);

		DecimalFormatSymbols europSymbols = new DecimalFormatSymbols(
				Locale.getDefault());
		europSymbols.setDecimalSeparator('.');
		DecimalFormat df = new DecimalFormat("0.0000", europSymbols);
		df.setMaximumFractionDigits(8);

		for (Iterator<ProfileConcept> iter = positiveConcepts.iterator(); iter
				.hasNext();) {
			ProfileConcept concept = iter.next();
			ArrayList<String> conc = concept.getConcepts();
			if (conc.get(0).contains(":")) {
				instancesBooleanMap.put(conc.get(0).split(":")[0], "false");
				max = concept.getWeight();
				if (concept.getWeight() > max)
					instancesWeightsMap.put(conc.get(0).split(":")[0],
							concept.getWeight());
				else
					instancesWeightsMap.put(conc.get(0).split(":")[0], max);
				entities.add(concept);
			} else if (conc.get(0).contains(" AND ")) {
				associationRules.add(concept);
			} else {
				simpleConcepts.add(concept);
			}
		}

		if ((!positiveConcepts.isEmpty()) && (!negativeConcepts.isEmpty()))
			userFile = userFile + "(AND ";

		if (!positiveConcepts.isEmpty()) {

			posConcepts = posConcepts + "(SOME has_interest (OR ";
			for (Iterator<ProfileConcept> iter = positiveConcepts.iterator(); iter
					.hasNext();) {
				ProfileConcept concept = iter.next();
				ArrayList<String> conc = concept.getConcepts();

				associationRulesNumber = 0;

				conc.trimToSize();
				if (conc.size() > 1) {
					posConcepts = posConcepts.concat("(AND ");
					for (Iterator<String> concElem = conc.iterator(); concElem
							.hasNext();) {

						@SuppressWarnings("unused")
						String currConcept = concElem.next();
					}
					posConcepts = posConcepts.concat(") ");
				} else {
					if (conc.get(0).contains(" AND ")) {
						posConcepts = posConcepts.concat(" profsubconcept"
								+ ++associationRulesNumber);
						weightConstructor += "(WEIGHT "
								+ "profsubconcept"
								+ associationRulesNumber
								+ " "
								+ df.format(concept
										.getCurrentNormalisedWeightLamdaVersion(
												Integer.parseInt(sessionsNumber),
												lamda)) + ")\n";
					} else if (conc.get(0).contains(":")) {
						String instance = conc.get(0).split(":")[0];

						// WEIGHT Constructor
						if (instancesBooleanMap.get(instance).equalsIgnoreCase(
								"false")) {
							weightConstructor += "(WEIGHT "
									+ "inst_"
									+ instance
									+ " "
									+ df.format(concept
											.getCurrentNormalisedWeightLamdaVersion(
													Integer.parseInt(sessionsNumber),
													lamda)) + ")\n";
							posConcepts = posConcepts.concat(" inst_"
									+ instance);
							instancesBooleanMap.put(instance, "true");

							// INSTANCE Constructor
							instanceConstructor += "(INSTANCE " + instance
									+ " inst_" + instance + " >= 0.1)\n";

							// RELATED Constructor
							relatedContentConstructor += "(RELATED content "
									+ instance + " has_interest)\n";
							relatedRuleConstructor += "(RELATED rule "
									+ instance + " inRule)\n";
						}
					} else {
						posConcepts = posConcepts.concat(" " + conc.get(0));
						weightConstructor += "(WEIGHT "
								+ conc.get(0)
								+ " "
								+ df.format(concept
										.getCurrentNormalisedWeightLamdaVersion(
												Integer.parseInt(sessionsNumber),
												lamda)) + ")\n";
					}
				}
			}
			posConcepts = posConcepts + ")) ";
			associationRulesNumber = 0;
		}

		if (!negativeConcepts.isEmpty()) {
			negConcepts = negConcepts + "(ALL has_interest (NOT (OR ";
			for (Iterator<ProfileConcept> enc = negativeConcepts.iterator(); enc
					.hasNext();) {
				ProfileConcept concept = enc.next();
				ArrayList<String> conc = concept.getConcepts();
				conc.trimToSize();
				if (conc.size() > 1) {
					negConcepts = negConcepts.concat("(AND ");
					for (Iterator<String> concElem = conc.iterator(); concElem
							.hasNext();) {
						@SuppressWarnings("unused")
						String currConcept = concElem.next();
					}
					negConcepts = negConcepts.concat(") ");
				} else {
					// negConcepts =
					// negConcepts.concat(linkProperty(conc.firstElement()) +
					// " ");
					// negConcepts = negConcepts.concat(" " +
					// conc.firstElement() + " ");
				}
			}
			negConcepts = negConcepts + "))) ";
		}

		userFile = userFile + posConcepts + negConcepts;
		if ((!positiveConcepts.isEmpty()) && (!negativeConcepts.isEmpty()))
			userFile = userFile + ")";
		userFile = userFile + " user" + userID + ")\n";

		for (Iterator<ProfileConcept> iter = associationRules.iterator(); iter
				.hasNext();) {
			associationRulesNumber++;
			ProfileConcept concept = iter.next();
			userFile += "(IMPLIES (AND";
			String[] rules = concept
					.getConcepts()
					.toString()
					.substring(1, concept.getConcepts().toString().length() - 1)
					.split("\\s+");
			for (int i = 0; i < rules.length; i++) {
				userFile += " (SOME inRule " + rules[i] + ")";
				i++;
			}
			userFile += ") profsubconcept" + associationRulesNumber + ")\n";
		}

		// IMPLIES CONSTRUCTORS (Entities)
		for (Iterator<ProfileConcept> iter = entities.iterator(); iter
				.hasNext();) {
			ProfileConcept concept = iter.next();
			ArrayList<String> conc = concept.getConcepts();
			instancesBooleanMap.put(conc.get(0).split(":")[0], "false");
			userFile += "(IMPLIES inst_"
					+ concept.getConcepts().toString().split(":")[0].replace(
							"[", "")
					+ " "
					+ concept.getConcepts().toString().split(":")[1].replace(
							"]", "") + ")" + "\n";

		}
		String userKRSSProfile = userFile + instanceConstructor
				+ disjointConstructor + weightConstructor
				+ relatedContentConstructor + relatedRuleConstructor
				+ relatedConstructor;
		System.out.println(userKRSSProfile);
		return userKRSSProfile;
	}

	/**
	 * Gets the number of interaction sessions per user.
	 *
	 * @param accountID the account id
	 * @param userID the user id
	 * @return the sessions number
	 */
	private String getSessionsNumber(String accountID, String userID) {
		URL url = null;
		try {
			url = new URL(Config.FULL_EXPORT_ROOT + accountID
					+ Config.GET_SESSIONS_NUMBER_SUFFIX + userID);
		} catch (MalformedURLException e2) {
			e2.printStackTrace();
		}
		URLConnection yc2 = null;
		try {
			yc2 = url.openConnection();
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		String userpass = "YOUTUBE-TEST" + ":" + "YOUTUBE-TEST";
		@SuppressWarnings("static-access")
		String basicAuth = "Basic "
				+ new String(new Base64().encode(userpass.getBytes()));
		yc2.setRequestProperty("Authorization", basicAuth);
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(yc2.getInputStream()));
		} catch (IOException e) {
			e.printStackTrace();
		}
		String inputLine;
		Vector<String> sessions = new Vector<String>();
		try {
			while ((inputLine = in.readLine()) != null) {

				sessions.add(inputLine);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		return sessions.toString().replaceAll("[^\\d.]", "");

	}

}
