/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>. 
 */
package test;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import onto.OntologyRetriever;
import profile.UserProfileConstructor;
import sys.Config;
import data.ProfileConcept;
import lifr.logic.firstorder.KnowledgeBase;

/**
 * Class testing the user profile creation. A user makes certain transactions,
 * w.r.t. a given number of content items that s/he has
 * supposedly consumed. The web pages and ads are collected randomly. A user
 * profile is formed according to these transactions and a corresponding KRSS
 * rule is created. Alternatively, a KRSS rule can be built from a set of
 * user-defined profile concepts for more controlled reasoner recommendation
 * testing.
 * 
 * @author Dorothea Tsatsou
 * @author Pantelis Ieronimakis
 */
public class TestUserProfileCreator {

	/**
	 * Instantiates a new test user profile creator.
	 */
	public TestUserProfileCreator() {

	}


	/**
	 * Constructs a new KRSS user profile - one can use a set of test concepts
	 * defined explicitly, or generate random transactions and feed them to the
	 * constructor.
	 * 
	 * @param userID
	 *            the username
	 * @param type
	 *            the KRSS profile type that this test will construct
	 */
	public static void createKRSSProfile(String userID, String accountID) {
		// PropertyRetriever.classifyProperties();
		KnowledgeBase kb = OntologyRetriever.getOntology();// getOntologyHierarchy();
		ArrayList<ProfileConcept> empty = new ArrayList<ProfileConcept>(1);
			ArrayList<ProfileConcept> concepts = ProfileConcept
					.readFromFile(Config.PROFILE_ROOT  + userID
							+ "_plain_profile.txt");
			
			for (int i = 0; i < concepts.size(); i++) {
				concepts.get(i)
						.setWeight(concepts.get(i).getNormalisedWeight());
				
			}
			UserProfileConstructor construct = new UserProfileConstructor(
					userID, concepts, empty);
			construct.constructProfile(kb);
	}
	
	
	/**
	 * Constructs a new KRSS user profile - one can use a set of test concepts
	 * defined explicitly, or generate random transactions and feed them to the
	 * constructor.
	 * 
	 * @param userID
	 *            the username
	 * @param type
	 *            the KRSS profile type that this test will construct
	 * @param lamda the time decay factor 
	 */
	public static void createKRSSProfile(String userID, String accountID,Double lamda) {
		// PropertyRetriever.classifyProperties();
		KnowledgeBase kb = OntologyRetriever.getOntology();// getOntologyHierarchy();
		ArrayList<ProfileConcept> empty = new ArrayList<ProfileConcept>(1);
			ArrayList<ProfileConcept> concepts = ProfileConcept
					.readFromFile(Config.PROFILE_ROOT  + userID
							+ "_plain_profile.txt");
			
			for (int i = 0; i < concepts.size(); i++) {
				concepts.get(i)
						.setWeight(concepts.get(i).getNormalisedWeight());
				
			}
			UserProfileConstructor construct = new UserProfileConstructor(
					userID, concepts, empty);
			construct.constructProfile(kb,lamda);
	}

	
	/**
	 * Creates a file containing the profile concepts for the given user as well
	 * as certain attributes for each profile concept (weight,
	 * positive/negative).
	 * 
	 * @param userId
	 *            the user id
	 */
	public void createProfileFormat(String userId) {
		ArrayList<ProfileConcept> concepts = ProfileConcept
				.readFromFile(Config.PROFILE_ROOT + "/" + userId
						+ "_concepts.txt");
		for (int i = 0; i < concepts.size(); i++) {
			concepts.get(i).setWeight(concepts.get(i).getNormalisedWeight());
		}
		ProfileConcept.writeToFileProfileFormat(concepts, Config.PROFILE_ROOT
				+ "/" + userId + "_profile.txt");

	}

	/**
	 * Initialization function for all files of the given user.
	 * 
	 * @param userID
	 *            the user id
	 */
	public void userInit(String userID) {
		String[] filepath = {}; 
		try {
			for (int i = 0; i < filepath.length; i++) {
				File file = new File(filepath[i]);
				if (file.exists())
					file.delete();
				file.createNewFile();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * The main method, that triggers testing of the user profile creator.
	 * 
	 * @param args
	 *            <i>No arguments
	 */
	public static void main(String[] args) {
		Double lamda = 0.000000000005;
		String user = "7";
		String accountID = "YOUTUBE-TEST";
//		test.userInit(user);
		createKRSSProfile(user, accountID,lamda);
	}

}
