/*
 * This file is part of Linked Profiler
 * Copyright (C) 2015 Pantelis Ieronimakis, Dorothea Tsatsou and Vasileios Mezaris.
 * 
 * Linked Profiler is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linked Profiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Linked Profiler.  If not, see <http://www.gnu.org/licenses/>.
 */
package transform;

import onto.*;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Scanner;
import java.util.Vector;

import data.GAINEntities;
import data.GAINFullExport;

import lifr.logic.firstorder.KnowledgeBase;
import lifr.logic.firstorder.Predicate;
import lifr.reasoner.ProofNotFoundException;
import lifr.reasoner.Reasoner;
import lifr.reasoner.krhyper.KrHyper;
import lifr.util.KRSSFuzzyParser;

/**
 * The Class transformLODtoLUMO.It provides the methods to retrieve concepts from the LUMO ontology 
 * based on given LOD entities, from several vocabularies (e.g. DBPedia, YAGO). Depends on available mappings 
 * in the LUMO mappings ontology. 
 * 
 *  @author Dorothea Tsatsou
 *  @author Pantelis Ieronimakis
 */
public class transformLODtoLUMO {
	
	/** The Constant reasonerTimeout. */
	protected static final int reasonerTimeout = 6000;
	
	/** The unsolved_memory. */
	protected static int unsolved_memory = 0;
	
	/** The unsolved_timeout. */
	protected static int unsolved_timeout = 0;
	
	/** The Constant mintermweight. */
	protected static final int mintermweight = 2;
	
	/** The refuted. */
	public static int refuted = 0;

	/**
	 * <i>Null Constructor.
	 */
	public transformLODtoLUMO() {
	}

	/**
	 * Static method that invokes the reasoner for the defined knowledge base.
	 * It matches a given ad to a given user profile.
	 *
	 * @param OntologyKB            the ontology.
	 * @return true if a match is found for the given ad in the specific user
	 *         profile.
	 */
	public static final Vector<Predicate> getModel(KnowledgeBase OntologyKB) {
		Vector<Predicate> reasonedModel = new Vector<Predicate>();
		KnowledgeBase kb = new KnowledgeBase();
		kb.setKnowledgeBase(OntologyKB);
		Reasoner reasoner = new KrHyper();

		reasoner.setKnowledgeBase(kb);

		@SuppressWarnings("unused")
		String s = kb.toString();

		boolean model = false;
		try {

			Runtime r = Runtime.getRuntime();
			long currentTime = System.currentTimeMillis();
			long initBytes = r.totalMemory() - r.freeMemory();

			model = reasoner.reason(mintermweight, mintermweight + 3,
					reasonerTimeout);

			long MemoryConsumed = r.totalMemory() - r.freeMemory() - initBytes;
			long time = System.currentTimeMillis() - currentTime;
			r.gc();

			if (model) {
				if (!reasoner.getModel().isEmpty()) {
					reasonedModel = reasoner.getModel();
				}
			} else {
				// System.out.println("Refutation found in "+time+
				// "ms.\nConsumed "+MemoryConsumed+" bytes.\n");
				refuted++;
			}
		} catch (ProofNotFoundException ex) {
			unsolved_timeout++;
			System.out
					.println("No Solution found for KRSS problem (Timeout)\n");
		} catch (Error err) {
			unsolved_memory++;
			System.out
					.println("No Solution found for KRSS problem (Out of Memory)\n");
		}
		kb.clear();
		return reasonedModel;
	}

	/**
	 * Translate entities from LOD vocabularies to LUMO based on LUMO mappings.
	 *
	 * @param concs the concepts
	 * @return the hash map
	 */
	public static HashMap<String, Double> translate(
			HashMap<String, Double> concs) {
		KnowledgeBase ontoKB = OntologyRetriever
				.getOntologyMappingsSubset(concs.keySet());

		HashMap<String, Double> interests = performTranslation(concs, ontoKB);
		HashMap<String, Double> lumoInterests = new HashMap<String, Double>();

		for (Iterator<String> e = interests.keySet().iterator(); e.hasNext();) {
			String s = e.next();
			if (s.contains("lumo#"))
				lumoInterests.put(s.replaceAll("lumo#", ""), interests.get(s));
		}

		// Collections.sort(all, Collections.reverseOrder());
		return lumoInterests;
	}

	/**
	 * Translate GAIN full export concepts per session.
	 *
	 * @param exp the GAIN export
	 * @return the concepts
	 */
	public static HashMap<String, Double> translateGAINFullExportConceptsPerSession(
			data.GAINFullExport exp) {
		HashMap<String, Double> concs = exp.getConcepts();
		return concs;
	}

	/**
	 * Perform translation via the LiFR semantic reasoner.
	 *
	 * @param concs the GAIN concepts
	 * @param ontoKB the ontology KB
	 * @return the model containing all mapped concepts
	 */
	public static HashMap<String, Double> performTranslation(
			HashMap<String, Double> concs, KnowledgeBase ontoKB) {
		HashMap<String, Double> model = new HashMap<String, Double>(30);

		KnowledgeBase factsKB = makeOntoFacts(concs);

		KnowledgeBase kb = new KnowledgeBase();
		kb.mergeKnowledgeBase(ontoKB);
		kb.mergeKnowledgeBase(factsKB);

		Vector<Predicate> matchMake = getModel(kb);
		for (Iterator<Predicate> iter = matchMake.iterator(); iter.hasNext();) {
			Predicate p = iter.next();
			if (!p.unary())
				continue;
			if (p.getDegree().getLowerBound().getDegree() < 0)
				continue;
			model.put(p.getName(), p.getDegree().getLowerBound().getDegree());
		}
		return model;
	}

	/**
	 * Make ontology facts. Utility method. 
	 *
	 * @param concepts the concepts
	 * @return the knowledge base
	 */
	private static KnowledgeBase makeOntoFacts(HashMap<String, Double> concepts) {
		String problem = "";
		for (Iterator<String> iter = concepts.keySet().iterator(); iter
				.hasNext();) {
			String concept = iter.next();
			if (concepts.get(concept) == 0)
				continue;
			problem += "(INSTANCE a " + concept + " >= "
					+ concepts.get(concept) + ")" + "\n";
		}

		KRSSFuzzyParser parser = new KRSSFuzzyParser(problem);
		KnowledgeBase kb = parser.getKB();
		return kb;
	}

	/**
	 * Translate GAIN latest full export.
	 *
	 * @param url the GAIN output url
	 * @return the translated entities
	 * @throws FileNotFoundException the file not found exception
	 */
	public static String translateGAINLatestFullExport(String url)
			throws FileNotFoundException {
		ArrayList<String> concepts = new ArrayList<String>();
		Vector<data.GAINFullExport> exp = input.readLatestFullExport
				.readGAINFullExport(url);
		List<data.GAINFullExport> newexp = new LinkedList<data.GAINFullExport>();
		ArrayList<String> clist = new ArrayList<String>();
		StringBuilder acc = new StringBuilder();
		for (int i = 0; i < exp.size(); i++) {
			data.GAINFullExport fexp = exp.get(i);
			HashMap<String, Double> concs = new HashMap<String, Double>();
			Iterator<Map.Entry<String, Double>> it = fexp.getConcepts()
					.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, Double> pairs = it.next();
				String inputConcept = pairs.getKey().replace(
						"_(disambiguation)", "");
				if (inputConcept.toLowerCase().startsWith("d_o")
						|| inputConcept.toLowerCase().startsWith("d_r")
						&& !inputConcept.contains("Thing")) {
					clist.add(inputConcept.substring(0,
							inputConcept.length() - 1));
					concs.put(
							"dbpedia#".concat(
									inputConcept.substring(4,
											inputConcept.length() - 1))
									.replaceAll("_(disambiguation)", ""), pairs
									.getValue());

				}
				if (inputConcept.toLowerCase().startsWith("c")
						&& !inputConcept.contains("Thing")) {
					concs.put(
							"dbpedia#".concat(
									inputConcept.substring(2,
											inputConcept.length() - 1))
									.replaceAll("_(disambiguation)", ""), 1.00);
					concepts.add(inputConcept.substring(2,
							inputConcept.length() - 1));
				}

			}
			HashMap<String, Double> newconcs = translate(concs);
			newexp.add(new data.GAINFullExport(fexp.getSessionID(), fexp
					.getObjectID(), fexp.getSubobjectID(), fexp.getInterest(),
					newconcs));

		}
		for (int i = 0; i < newexp.size(); i++) {
			if (i == 0) {
				acc.append("[");
			}
			LinkedHashSet<String> thelist = new LinkedHashSet<String>();
			HashSet hs = new HashSet();
			hs.addAll(clist);
			clist.clear();
			clist.addAll(hs);

			acc.append("{\"sessionId\":" + "\"" + newexp.get(i).getSessionID()
					+ "\"" + ",");
			acc.append("\"objectId\":" + "\"" + newexp.get(i).getObjectID()
					+ "\"" + ",");
			acc.append("\"parentObjectId\":" + newexp.get(i).getSubobjectID()
					+ ",");
			if (newexp.get(i).getConcepts().size() != 0) {
				acc.append("\"interest\":" + "\"" + newexp.get(i).getInterest()
						+ "\"" + ",");
			} else {
				acc.append("\"interest\":" + "\"" + newexp.get(i).getInterest()
						+ "\"");
			}

			Iterator<Map.Entry<String, Double>> itt = newexp.get(i)
					.getConcepts().entrySet().iterator();
			while (itt.hasNext()) {
				Map.Entry<String, Double> pairs = itt.next();
				if (itt.hasNext()) {
					acc.append("\"" + "l_o_" + pairs.getKey() + "\"" + ":"
							+ "\"" + pairs.getValue() + "\"" + ",");
				} else {
					acc.append("\"" + "l_o_" + pairs.getKey() + "\"" + ":"
							+ "\"" + pairs.getValue() + "\"");
				}

			}
			if (i != newexp.size() - 1) {
				acc.append("},\n");
			} else {
				acc.append("}]\n");
			}
		}

		return acc.toString();
	}

	/**
	 * Translate GAIN latest full export list.
	 *
	 * @param url the url
	 * @return the of translated concepts
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public static List<GAINFullExport> translateGAINLatestFullExportList(
			String url) throws IOException {
		ArrayList<String> concepts = new ArrayList<String>();
		Vector<data.GAINFullExport> exp = input.readLatestFullExport
				.readGAINFullExport(url);
		List<data.GAINFullExport> newexp = new LinkedList<data.GAINFullExport>();
		ArrayList<String> clist = new ArrayList<String>();
		StringBuilder acc = new StringBuilder();
		for (int i = 0; i < exp.size(); i++) {
			data.GAINFullExport fexp = exp.get(i);
			HashMap<String, Double> concs = new HashMap<String, Double>();
			Iterator<Map.Entry<String, Double>> it = fexp.getConcepts()
					.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, Double> pairs = it.next();
				String inputConcept = pairs.getKey().replace(
						"_(disambiguation)", "");
				if (inputConcept.toLowerCase().startsWith("d_o")
						|| inputConcept.toLowerCase().startsWith("d_r")
						&& !pairs.getKey().contains("Thing")) {

					clist.add(inputConcept.substring(0,
							inputConcept.length() - 1));
					concs.put("dbpedia#".concat(inputConcept.substring(4,
							inputConcept.length() - 1)), pairs.getValue());

				}
				if (inputConcept.toLowerCase().startsWith("c")
						&& !pairs.getKey().contains("Thing")) {
					concs.put("context#".concat(inputConcept.substring(2,
							inputConcept.length() - 1)), 1.00);
				}

			}

			HashMap<String, Double> newconcs = translate(concs);

			newexp.add(new data.GAINFullExport(fexp.getSessionID(), fexp
					.getObjectID(), fexp.getSubobjectID(), fexp.getInterest(),
					newconcs));

		}

		for (int i = 0; i < newexp.size(); i++) {
			if (i == newexp.size() - 1) {

			}
			LinkedHashSet<String> thelist = new LinkedHashSet<String>();
			HashSet hs = new HashSet();
			hs.addAll(clist);
			clist.clear();
			clist.addAll(hs);

			acc.append("{\"sessionId\":" + "\"" + newexp.get(i).getSessionID()
					+ "\"" + ",");
			acc.append("\"objectId\":" + "\"" + newexp.get(i).getObjectID()
					+ "\"" + ",");
			acc.append("\"parentObjectId\":" + newexp.get(i).getSubobjectID()
					+ ",");
			acc.append("\"interest\":" + "\"" + newexp.get(i).getInterest()
					+ "\"" + ",");

			Iterator<Map.Entry<String, Double>> itt = newexp.get(i)
					.getConcepts().entrySet().iterator();
			while (itt.hasNext()) {
				Map.Entry<String, Double> pairs = itt.next();
				if (itt.hasNext()) {
					acc.append("\"" + "l_o_" + pairs.getKey() + "\"" + ":"
							+ "\"" + pairs.getValue() + "\"" + ",");
				} else {
					acc.append("\"" + "l_o_" + pairs.getKey() + "\"" + ":"
							+ "\"" + pairs.getValue() + "\"");
				}

			}
			acc.append("}\n");
		}
		return newexp;
	}

	/**
	 * Translate GAIN entities.
	 *
	 * @param url the GAIN output url
	 * @return the translated entities
	 */
	public static String translateGAINEntities(String url) {
		ArrayList<String> concepts = new ArrayList<String>();
		List<GAINEntities> exp = input.readGainEntities.readGainentities(url);
		ArrayList<GAINEntities> geexp = new ArrayList<GAINEntities>();
		for (int i = 0; i < exp.size(); i++) {
			data.GAINEntities gexp = exp.get(i);
			ArrayList<data.GAINEntities.Entities> test = gexp.getEntities();
			ArrayList<data.GAINEntities.Entities> keep = new ArrayList<data.GAINEntities.Entities>();
			data.GAINEntities ge = new GAINEntities();

			for (int j = 0; j < test.size(); j++) {
				
				HashMap<String, Double> concs = new HashMap<String, Double>();
				if ((test.get(j).getEntityType().equals("named entity") && test
						.get(j).getIType() != null)
						&& test.get(j).getIType() != "Thing") {
					String inputLine = test.get(j).getIType()
							.replaceAll("_(disambiguation)", "");
					concs.put(
							"dbpedia#".concat(inputLine.substring(test.get(j)
									.getIType().lastIndexOf("/") + 1)), 1.000);
					concepts.add(inputLine.substring(test.get(j).getIType()
							.lastIndexOf("/") + 1));
					HashMap<String, Double> newconcs = translate(concs);
					if (newconcs.size() != 0) {
						keep.add(ge.new Entities(test.get(j).getEntitySource(),
								test.get(j).getEntityLod(), test.get(j)
										.getIType(), test.get(j)
										.getEntityLabel(), test.get(j)
										.getEntityTypeLabel(), test.get(j)
										.getEntityType(), test.get(j)
										.getEntityConfidence(), test.get(j)
										.getEntityRelevance(), newconcs));
					}
				}
			}

			if (keep.size() > 0) {
				ge.setObjectID(exp.get(i).getObjectID());
				data.GAINEntities.Attributes atts = ge.new Attributes();
				atts.setAttributeStart(exp.get(i).getAttributes()
						.getAttributeStart());
				atts.setAttributeEnd(exp.get(i).getAttributes()
						.getAttributeEnd());
				ge.setAttributes(atts);
				ge.setAccountID(exp.get(i).getAccountID());
				ge.setID(exp.get(i).getID() + "");
				ge.setV(exp.get(i).getV());
				ge.setEntities(keep);
				ge.setRating(exp.get(i).getRating());
				ge.setGroup(exp.get(i).getGroup());
				ge.setType(exp.get(i).getType());
				ge.setParentObjectId(exp.get(i).getParentObjectId());
				geexp.add(ge);
			}
		}

		List<Map<String, Object>> list = new LinkedList<Map<String, Object>>();
		for (int i = 0; i < geexp.size(); i++) {

			Map<String, Object> map = new LinkedHashMap<String, Object>();
			Map<String, Double> map2 = new LinkedHashMap<String, Double>();
			Map<String, Object> map3 = new LinkedHashMap<String, Object>();

			map.put("objectID", geexp.get(i).getObjectID());
			map2.put("start", geexp.get(i).getAttributes().getAttributeStart());
			map2.put("end", geexp.get(i).getAttributes().getAttributeEnd());
			map.put("attributes", map2);
			map.put("accountId", geexp.get(i).getAccountID());
			map.put("_id", geexp.get(i).getID());
			map.put("__v", geexp.get(i).getV());

			for (int j = 0; j < geexp.get(i).getEntities().size(); j++) {
				for (Entry<String, Double> entry : geexp.get(i).getEntities()
						.get(j).getNewconcs().entrySet()) {
					map3.put(
							"LUMOtype",
							entry.getKey()
									.toString()
									.substring(
											entry.getKey().toString()
													.indexOf("#") + 1)
									.substring(0, 1).toUpperCase()
									+ entry.getKey()
											.toString()
											.substring(
													entry.getKey().toString()
															.indexOf("#") + 1)
											.substring(1));
					map3.put("label", geexp.get(i).getEntities().get(j)
							.getEntityLabel());
					map3.put("entityType", geexp.get(i).getEntities().get(j)
							.getEntityType());
					map3.put("confidence", geexp.get(i).getEntities().get(j)
							.getEntityConfidence());
					map3.put("relevance", geexp.get(i).getEntities().get(j)
							.getEntityRelevance());
					map.put("entities", map3);
				}
			}

			map.put("rating", geexp.get(i).getRating());
			map.put("group", geexp.get(i).getGroup());
			map.put("type", geexp.get(i).getType());
			map.put("parentObjectId", geexp.get(i).getParentObjectId() + "\n");

			list.add(map);

		}
		return String.valueOf(list);
	}

	/**
	 * Translate gain entities list.
	 *
	 * @param url the url
	 * @return the list
	 */
	public static List<GAINEntities> translateGAINEntitiesList(String url) {
		ArrayList<String> concepts = new ArrayList<String>();
		List<GAINEntities> exp = input.readGainEntities.readGainentities(url);
		ArrayList<GAINEntities> geexp = new ArrayList<GAINEntities>();

		for (int i = 0; i < exp.size(); i++) {

			data.GAINEntities gexp = exp.get(i);
			ArrayList<data.GAINEntities.Entities> test = gexp.getEntities();
			ArrayList<data.GAINEntities.Entities> keep = new ArrayList<data.GAINEntities.Entities>();
			data.GAINEntities ge = new GAINEntities();
			for (int j = 0; j < test.size(); j++) {
				
				HashMap<String, Double> concs = new HashMap<String, Double>();
				if ((test.get(j).getEntityType().equals("named entity")
						&& test.get(j).getIType() != null && test.get(j)
						.getIType() != "Thing")) {
					String inputLine = test.get(j).getIType()
							.replaceAll("_(disambiguation)", "");
					concs.put(
							"dbpedia#".concat(inputLine.substring(test.get(j)
									.getIType().lastIndexOf("/") + 1)), 1.000);
					concepts.add(inputLine.substring(test.get(j).getIType()
							.lastIndexOf("/") + 1));
					HashMap<String, Double> newconcs = translate(concs);

					if (newconcs.size() != 0) {
						keep.add(ge.new Entities(test.get(j).getEntitySource(),
								test.get(j).getEntityLod(), test.get(j)
										.getIType(), test.get(j)
										.getEntityLabel(), test.get(j)
										.getEntityTypeLabel(), test.get(j)
										.getEntityType(), test.get(j)
										.getEntityConfidence(), test.get(j)
										.getEntityRelevance(), newconcs));
					}
				}
			}

			if (keep.size() > 0) {
				ge.setObjectID(exp.get(i).getObjectID());
				data.GAINEntities.Attributes atts = ge.new Attributes();
				atts.setAttributeStart(exp.get(i).getAttributes()
						.getAttributeStart());
				atts.setAttributeEnd(exp.get(i).getAttributes()
						.getAttributeEnd());
				ge.setAttributes(atts);
				ge.setAccountID(exp.get(i).getAccountID());
				ge.setID(exp.get(i).getID() + "");
				ge.setV(exp.get(i).getV());
				ge.setEntities(keep);
				ge.setRating(exp.get(i).getRating());
				ge.setGroup(exp.get(i).getGroup());
				ge.setType(exp.get(i).getType());
				ge.setParentObjectId(exp.get(i).getParentObjectId() + "\n");
				geexp.add(ge);
			}
		}

		List<Map<String, Object>> list = new LinkedList<Map<String, Object>>();
		for (int i = 0; i < geexp.size(); i++) {
			Map<String, Object> map = new LinkedHashMap<String, Object>();
			Map<String, Double> map2 = new LinkedHashMap<String, Double>();
			Map<String, Object> map3 = new LinkedHashMap<String, Object>();

			map.put("objectID", geexp.get(i).getObjectID());
			map2.put("start", geexp.get(i).getAttributes().getAttributeStart());
			map2.put("end", geexp.get(i).getAttributes().getAttributeEnd());
			map.put("attributes", map2);
			map.put("accountId", geexp.get(i).getAccountID());
			map.put("_id", geexp.get(i).getID());
			map.put("__v", geexp.get(i).getV());

			for (int j = 0; j < geexp.get(i).getEntities().size(); j++) {

				for (Entry<String, Double> entry : geexp.get(i).getEntities()
						.get(j).getNewconcs().entrySet()) {
					map3.put(

							"LUMOtype",
							entry.getKey()
									.toString()
									.substring(
											entry.getKey().toString()
													.indexOf("#") + 1)
									.substring(0, 1).toUpperCase()
									+ entry.getKey()
											.toString()
											.substring(
													entry.getKey().toString()
															.indexOf("#") + 1)
											.substring(1));
					map3.put("label", geexp.get(i).getEntities().get(j)
							.getEntityLabel());
					map3.put("entityType", geexp.get(i).getEntities().get(j)
							.getEntityType());
					map3.put("confidence", geexp.get(i).getEntities().get(j)
							.getEntityConfidence());
					map3.put("relevance", geexp.get(i).getEntities().get(j)
							.getEntityRelevance());
					map.put("entities", map3);
				}
			}

			map.put("rating", geexp.get(i).getRating());
			map.put("group", geexp.get(i).getGroup());
			map.put("type", geexp.get(i).getType());
			map.put("parentObjectId", geexp.get(i).getParentObjectId() + "\n");

			list.add(map);

		}

		return geexp;
	}

}
